@@ -1,20 +1,31 @@
 from user_agent import generate_user_agent
 from json.decoder import JSONDecodeError
 from requests import Session, Response
+from collections import OrderedDict
+from urllib.parse import urljoin
 from bs4 import BeautifulSoup
+from copy import deepcopy
 
 import json
 
 from .logger import Logger
+from .url import URL
 
 
 class FractifClient(Session):
+    _urls = None
     ua: str = generate_user_agent()
     soup: BeautifulSoup = None
     json: dict = None
     res_type: str = None  # 'json' or 'html'
 
-    def __init__(self, BASEURL: str = None, debug: bool = True):
+    def __init__(
+        self,
+        BASEURL: str = None,
+        debug: bool = True,
+        *args,
+        **kwargs,
+    ):
         Session.__init__(self)
 
         self.BASEURL = BASEURL
@@ -31,6 +42,24 @@ class FractifClient(Session):
             'response': self.parse_response,
         }
 
+        self.page = None
+
+        def is_property(attr):
+            v = getattr(type(self), attr, None)
+            return hasattr(v, '__get__') or hasattr(v, '__set__')
+
+        attrs = [
+            (attr, getattr(self, attr))
+            for attr in dir(self) if not is_property(attr)
+        ]
+        attrs = [v for v in attrs if isinstance(v[1], URL)]
+        attrs.sort(key=lambda v: v[1]._creation_counter)
+        self._urls = OrderedDict(deepcopy(attrs))
+        for k, v in self._urls.items():
+            setattr(self, k, v)
+        for url in self._urls.values():
+            url.browser = self
+
     def __str__(self) -> str:
         return f'<FractifClient {self.headers["User-Agent"]}>'
 
@@ -78,11 +107,9 @@ class FractifClient(Session):
         Arguments are optional parameters for url.
         >>> self.go('https://example.com')
         """
-        if not url.startswith('https://'):
-            url = self.BASEURL + url
         res = self.request(
-            method,
-            url,
+            method=method,
+            url=url,
             data=data,
             headers=headers,
             params=params,
@@ -94,3 +121,10 @@ class FractifClient(Session):
         )
         res.raise_for_status()
         return res
+
+    def absurl(self, uri, base=None):
+        if not base:
+            base = self.url
+        if base is None or base is True:
+            base = self.BASEURL
+        return urljoin(base, uri)
